import Flutter
import UIKit
import Smooch


public class SwiftSmoochPlugin: NSObject, FlutterPlugin {
    let serialQueue = DispatchQueue(label: "initializeBot")

    var channel : FlutterMethodChannel?

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "smooch_plugin", binaryMessenger: registrar.messenger())

        let instance = SwiftSmoochPlugin()
        instance.channel = channel

        registrar.addApplicationDelegate(instance)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if call.method == "init" {
            initialize(call.arguments, result: result)
            return
        }

        if call.method == "show" {
            show(call.arguments, result: result)
            return
        }

        if call.method == "logout" {
            logout(result: result)
            return
        }

        if call.method == "minimize" {
            minimize(call.arguments, result: result)
            return
        }

        result(FlutterMethodNotImplemented)
    }

}

extension SwiftSmoochPlugin {

    // MARK: - FlutterApplicationLifeCycleDelegate
    public func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        SmoochHandler.shared().registerPushToken(pushToken: deviceToken)
    }

    public func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable : Any], fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool {
        SmoochHandler.handlePushNotification(userInfo: userInfo)
        channel?.invokeMethod("onPushNotification", arguments: userInfo)
        completionHandler(.noData)
        return true
    }

    // MARK: - UNUserNotificationCenterDelegate
    public func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        NSLog("[SMOOCH] %@", #function)
    }

    public func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        NSLog("[SMOOCH] %@", #function)
    }

}

// MARK: - FlutterMethodCalls
extension SwiftSmoochPlugin {

    private func initialize(_ arguments: Any?, result: @escaping FlutterResult) {

        guard let arguments = arguments as? [String: Any] else {
            result(-1)
            return
        }

        if let properties = arguments["properties"] as? [String: Any] {
            SmoochHandler.shared().properties = properties
        }

        SmoochHandler.shared().setMenuItemsAllowed(arguments["menuItemsAllowed"])
        SmoochHandler.shared().setChatColor(arguments["chatColor"])

        guard let appId = arguments["appId"] as? String,
            let integrationId = arguments["integrationId"] as? String,
            let userId = arguments["userId"] as? String,
            let smoochToken = arguments["smoochToken"] as? String else {
                return
        }

        SmoochHandler.shared().app_id = appId
        SmoochHandler.shared().integration_id = integrationId
        SmoochHandler.shared().smooch_token = smoochToken

        let settings = SKTSettings(integrationId: integrationId)
        if let chatColor = SmoochHandler.shared().chatColor {
            settings.conversationAccentColor = chatColor
        }

        if let userTextColor = arguments["userTextColor"] as? String {
            settings.userMessageTextColor = UIColor(hex: userTextColor)
        }

        settings.conversationStatusBarStyle = .lightContent;
        settings.allowedMenuItems = SmoochHandler.shared().menuItensAllowed
        settings.enableAppDelegateSwizzling = false
        settings.enableUserNotificationCenterDelegateOverride = false

        Smooch.destroy()

        self.initWith(settings: settings) { (error) in
            if let error = error {
                print(error.localizedDescription)
                result(-1)
                return
            }
        }

        self.loginWith(userId: userId, smoochToken: smoochToken) { (error, userInfo) in
            if let loginError = error {
                print(loginError.localizedDescription)
            }
            
            if let loginInfo = userInfo {
                print(loginInfo)
            }
            
            SmoochHandler.shared().smoochInited = true
            SmoochHandler.shared().conversation = Smooch.conversation()
            result(0)
        }
    }

    private func initWith(settings: SKTSettings, completion: @escaping(Error?) -> Void) {
        serialQueue.async {
            Smooch.initWith(settings) { (error, dictionary) in
                if let error = error {
                    completion(error)
                    return
                }

                completion(nil)
            }
        }
    }

    private func loginWith(userId: String, smoochToken: String, completion: @escaping(Error?, [AnyHashable : Any]?) -> Void) {
        serialQueue.async {
            Smooch.login(userId, jwt: smoochToken) { (error, userInfo) in
                if let error = error {
                    completion(error, nil)
                    return
                }

                if let userInfo = userInfo {
                    completion(nil, userInfo)
                    return
                }
                
                completion(SmoochError.description("(Smooch) Unexpected Login error..."), nil)
            }
        }
    }

    private func show(_ arguments: Any?, result: @escaping FlutterResult) {
        if let arguments = arguments as? [String: Any] {
            let language = ((arguments["lang"] as? String) ?? "").completeAcronym
            
            SmoochHandler.shared().setMenuItems(arguments["accessory"])
            SmoochHandler.shared().language = language
        }

        SmoochHandler.shared().show(channel) { (res) in
            result(res)
        }
    }

    private func logout(result: @escaping FlutterResult) {
        SmoochHandler.shared().logout { resultStatus in
            result(resultStatus)
        }
    }

    private func minimize(_ arguments: Any?, result: @escaping FlutterResult) {

        if let arguments = arguments {
            print("FlutterMethod \(#function) arguments: \(arguments)")
        }

        SmoochHandler.shared().closeConversarionViewController()
        result(nil)
    }

}

enum SmoochError: Error {
    case description(_ message: String)
}
